from excel2meta_interface.utils import messages
import logging
import jinja2
import json


class UtilsHelper:
    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)

    def __init__(self):
        self.jinja_environment = None

    def get_jinja_template(self, template_directory, template_name):
        module = __name__ + ".get_jinja_template"
        if template_name is None:
            return messages.message["jinja_template_name_not_provided"], None

        the_template = None
        self.logger.debug(module, "Jinja template requested: >%s<" % template_name)
        self.logger.debug(module, "Jinja template directory: >%s<" % template_directory)
        try:
            self.jinja_environment = jinja2.Environment(loader=jinja2.FileSystemLoader(template_directory))
            self.jinja_environment.filters["safe"] = json.dumps
            the_template = self.jinja_environment.get_template(template_name)
            self.logger.debug(module + ": Found jinja template: " + template_name)
        except jinja2.exceptions.TemplateNotFound:
            self.logger.error(module + ": Could not find jinja template >" + template_name
                            + "< in directory >" + template_directory + "<.")
            return messages.message["jinja_template_not_found"], None

        return messages.message["ok"], the_template



import csv
import glob
import json
import re
from datetime import datetime
from os import path, makedirs
from uuid import uuid4

from excel2meta_interface.utils import messages, helpers


class Reference2Interface:
    """
        Reference2Interface
        Convert reference files, generated by e.g. Excel2Reference, to defined json schema files
            as per https://github.com/jacbeekers/metadata-registry-interface-specifications
    """
    start_time = datetime.now()
    # used the create sub directories
    one_on_one_name = "one-on-one"

    def __init__(self, configuration_file="resources/excel2json_config.json"):
        # default values
        self.json_directory = "out/"
        self.json_file_prefix = "generated_"
        self.schema_base_directory = "resources/metadata-registry-interface-specifications"
        self.version = "0.2.0"
        # within schema_directory:
        self.schema_location = "metadata/schemas/interface/" + self.version
        self.result = messages.message["ok"]
        self.data_sources = []
        self.edc_lineage_column_header = [
            "Association",
            "From Connection",
            "To Connection",
            "From Object",
            "To Object",
        ]
        try:
            with open(configuration_file) as config:
                data = json.load(config)
                self.json_directory = data["json_directory"]
                if "json_file_prefix" in data:
                    self.json_file_prefix = data["json_file_prefix"]
                if "schema_base_directory" in data:
                    self.schema_base_directory = data["schema_base_directory"]
                if "version" in data:
                    self.version = data["version"]
                if "schema_location" in data:
                    self.schema_location = data["schema_location"]
                if "jinja_templates" in data:
                    self.jinja_templates = data["jinja_templates"]
                else:
                    self.jinja_templates = "resources/jinja_templates"
                if "lineage_output" in data:
                    self.lineage_output = data["lineage_output"]
                else:
                    self.lineage_output = "out/"
                if "json_file_prefix" in data:
                    self.json_file_prefix = data["json_file_prefix"]
                else:
                    self.json_file_prefix = self.start_time.isoformat(timespec="microseconds").replace(':', '-')
                if self.json_file_prefix == "TIMESTAMP":
                    self.json_file_prefix = self.start_time.isoformat(timespec="microseconds").replace(':', '-')
                self.json_file_prefix += "--"
                if "one_on_ones" in data:
                    self.one_on_ones = data["one_on_ones"]
                else:
                    self.one_on_ones = None

        except FileNotFoundError as e:
            print("Configuration file >" + configuration_file + "< not found.")
            self.result = messages.message["main_config_not_found"]

        self.one_on_patterns = []
        if self.one_on_ones is not None:
            with open(self.one_on_ones) as one_on_ones:
                patterns = json.load(one_on_ones)["patterns"]
            for pattern in patterns:
                regex_zone_pattern = re.compile(pattern["pattern"])
                label = pattern["label"]
                self.one_on_patterns.append(dict(
                      zone_pattern=regex_zone_pattern
                    , label=label
                    , source_edc_resource=pattern["source_edc_resource"]
                    , source_edc_datasource=pattern["source_edc_datasource"]
                    , source_folder=pattern["source_folder"]
                    , target_edc_resource=pattern["target_edc_resource"]
                    , target_edc_datasource=pattern["target_edc_datasource"]
                ))
        self.one_on_one_data_sources = []
        self.helper = helpers.UtilsHelper()

    def process_generated_files(self):
        mapping_files = []
        reference_files = []
        attribute_files = []
        result = messages.message["ok"]

        # First process all REF files as some objects will get their uuid from it
        for file in glob.glob(self.json_directory + "20*--REF_-_*.json"):
            print("Processing reference file >%s<" % file)
            result = self.process_reference_file(file)
            if result["code"] == "OK":
                reference_files.append(file)

        # Generate attribute files for entities
        # It is assumed that reference_files has already been populated so the uuid of the entity can be found
        for file in glob.glob(self.json_directory + "20*--ATTRIB_-_*.json"):
            print("Processing attributes file >%s<" % file)
            result = self.process_attributes_file(file)
            if result["code"] == "OK":
                attribute_files.append(file)

        # Generate the association files
        for file in glob.glob(self.json_directory + "20*--MAP_-_*.json"):
            print("Processing mapping file >%s<" % file)
            result = self.process_mapping_file(file)
            if result["code"] == "OK":
                mapping_files.append(file)

        return result

    def process_attributes_file(self, attributes_file):
        result = messages.message["ok"]
        result_entity, jinja_attributes = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                                         , template_name="ref_entity_attributes.jinja")
        result_attribute, jinja_attribute = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                                           , template_name="ref_attribute.jinja")
        if result_attribute["code"] == "OK" and result_entity["code"] == "OK":
            with open(attributes_file, "r") as input:
                sources = json.load(input)
                for source in sources:
                    the_attributes = []
                    for attribute in source["attributes"]:
                        desc = attribute["attribute_description"].replace('"', '\\"')
                        attrib_content = jinja_attribute.render(
                              attribute_uuid=uuid4()
                            , attribute_name=attribute["attribute_name"]
                            , attribute_description=desc
                        )
                        the_attributes.append(json.loads(attrib_content))
                    # fill in the jinja template for attributes
                    the_uuid = self.get_entity_uuid(source["dataset"], data_sources=self.data_sources)
                    content = jinja_attributes.render(
                          attributes=json.dumps(the_attributes)
                        , entity_uuid=the_uuid
                        , version=self.start_time.strftime("%Y.%m.%d")
                    )
                    # print(content)
                    json_content = json.loads(content)
                    # print("content is >%s<" % json_content)
                    file_name = path.join(self.lineage_output, source["dataset"] + "_attributes.json")
                    with open(file_name, "w", encoding="utf8") as out:
                        json.dump(json_content, out, indent=4)
                    # if the data sources is in a one-on-one layer, also generate an attribute json file for the predecessor
                    if source["is_source"] == "True" and self.one_on_ones is not None:
                        for pattern in self.one_on_patterns:
                            # print("Checking for one-on-one", item["label"])
                            # zone_pattern = re.compile(pattern["pattern"])
                            the_zone = pattern["zone_pattern"].search(source["src_zone"])
                            if the_zone is None:
                                continue
                            else:
                                # print("try to jsonize:", content)
                                result = self.process_one_on_one_attributes(the_zone.group(), source
                                                                 , pattern["label"], the_attributes)
                                if result["code"] != "OK":
                                    return result

        else:
            result = messages.message["jinja_template_not_found"]
        return result

    def get_entity_uuid(self, entity, data_sources):
        # look in self.data_sources for the uuid of the entity
        count = 0
        entity_uuid = None
        for source in data_sources:
            # print("looking for entity_uuid in", source)
            if "dataset" in source["data_source"] and source["data_source"]["dataset"] == entity:
                entity_uuid = source["uuid"]
                count += 1
                # print("get_entity_uuid - found #%d entity >%s<. its uuid is >%s<" % (count, entity, source["uuid"]))

        if count == 0:
            print("Entity not found")
        elif count > 1:
            print("ERROR: Entity found more than once")

        return entity_uuid

    def process_mapping_file(self, mapping_file):
        result = messages.message["ok"]
        if "data-sources-" in mapping_file:
            result = self.process_map_data_source_file(mapping_file)
        elif "zones-" in mapping_file:
            result = self.process_map_zone_file(mapping_file)
        else:
            print("%s: This is not the file we are looking for..." % mapping_file)
        return result

    def process_reference_file(self, reference_file):
        result = messages.message["ok"]
        if "data-sources-" in reference_file:
            if "data-sources-source" in reference_file:
                is_source = True
            else:
                is_source = False
            result = self.process_ref_data_source_file(reference_file, is_source)
        elif "zones-" in reference_file:
            result = self.process_ref_zone_file(reference_file)
        else:
            print("%s: This is not the file we are looking for..." % reference_file)
        return result

    def process_ref_data_source_file(self, data_source_file, is_source=True):
        with open(data_source_file, "r") as input:
            sources = json.load(input)
            # print(sources)

        result, jinja_template = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                                , template_name="ref_data_source.jinja")
        if result["code"] != "OK":
            return result

        for data_source in sources:
            the_uuid = uuid4()
            data_source["description"] = data_source["description"].replace('"',"'")
            self.data_sources.append(dict(uuid=the_uuid, data_source=data_source))
            print("adding source to data_sources list:", data_source)
            # some string contain unescaped quotes with which json.loads generates an error
            # so need to change the description field accordingly
            desc = data_source["description"]
            # if "\"" in desc:
            #    # print("There's a double-quote in the description field.")
            #    desc = data_source["description"].replace('"', '\\"')
            #    # print("Fix: >%s<" % desc)
            content = jinja_template.render(
                entity_type="file"
                , uuid=the_uuid
                , name=data_source["dataset"]
                , version=self.start_time.strftime("%Y.%m.%d")
                , meta_version="0.2.0"
                , description=desc
                , set_id=str(data_source["id"])
                , app_id=data_source["app"]
            )
            if is_source:
                sub_folder = "sources"
                directory = path.join(self.lineage_output, sub_folder, data_source["src_zone"])
            else:
                sub_folder = "targets"
                directory = path.join(self.lineage_output, sub_folder, data_source["tgt_zone"])
            makedirs(directory, exist_ok=True)
            # create entity json
            file_name = path.join(directory, data_source["dataset"] + "_entity.json")
            with open(file_name, "w", encoding="utf8") as out:
                out.write(content)
                # print("Content is >%s<" % content)
            # create a copy in the sub folder source or target
            file_name = path.join(self.lineage_output, data_source["dataset"] + "_entity")
            with open(file_name + ".json", "w", encoding="utf8") as out:
                out.write(content)
            with open(file_name + ".meta", "w", encoding="utf8") as out:
                if is_source:
                    out.write("is_source=True\n")
                    out.write("zone=" + data_source["src_zone"] + "\n")
                else:
                    out.write("is_source=False\n")
                    out.write("zone=" + data_source["tgt_zone"] + "\n")
            # if the data sources is in a one-on-one layer, also generate a dataset json file for the predecessor
            if is_source and self.one_on_ones is not None:
                todo = True
                for pattern in self.one_on_patterns:
                    # print("Checking for one-on-one", item["label"])
                    # zone_pattern = re.compile(pattern["pattern"])
                    the_zone = pattern["zone_pattern"].search(data_source["src_zone"])
                    if the_zone is None:
                        continue
                    else:
                        # print("try to jsonize:", content)
                        result = self.process_one_on_one(the_zone.group(), data_source["dataset"]
                                                         , pattern["label"], json.loads(content))
                        if result["code"] != "OK":
                            return result
                    if todo:
                        todo = False
                        result = self.generate_configuration_file(self.lineage_output, pattern, data_source)

        return result

    def process_one_on_one(self, zone, source, label, original_content):
        """
            Generate the necessary files to generate a one-on-one mapping between a source and target
        :param zone: (not used)
        :param source:
        :param label:
        :param original_content:
        :return:
        """
        # print("Processing one-on-one for source >%s< in zone >%s<, label is >%s<" % (source, zone, label))

        # get the jinja2 template
        result, jinja_template = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                                , template_name="ref_data_source.jinja")
        if result["code"] != "OK":
            return result

        # print("original_content:", original_content)
        the_uuid = uuid4()
        # Add only if we have all the info. For now: don't
        the_source = dict(is_source="1:1", dataset=source)
        self.one_on_one_data_sources.append(dict(uuid=the_uuid, data_source=the_source))
        content = jinja_template.render(
            entity_type="file"
            , uuid=the_uuid
            , name=label + "-" + source
            , version=self.start_time.strftime("%Y.%m.%d")
            , meta_version="0.2.0"
            , description=original_content["description"]
            , set_id=str(original_content["dataset_id"])
            , app_id=original_content["application_id"]
        )

        directory = path.join(self.lineage_output, self.one_on_one_name)
        makedirs(directory, exist_ok=True)
        file_name = path.join(directory, label + "-" + source + "-1on1_entity")
        with open(file_name + ".json", "w", encoding="utf8") as out:
            out.write(content)

        return result

    def process_one_on_one_attributes(self, zone, source, label, attributes):
        """
         Creates the attribute json for the 1:1 datasource
        :param zone:
        :param data_set:
        :param label:
        :param attributes:
        :return:
        """
        result =  messages.message["ok"]
        result_entity, jinja_attributes = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                                         , template_name="ref_entity_attributes.jinja")
        if result_entity["code"] != "OK":
            return result_entity

        result_attribute, jinja_attribute = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                                           , template_name="ref_attribute.jinja")
        if result_attribute["code"] != "OK":
            return result_attribute

        # generate new uuids for the 1:1
        the_attributes = []
        for attribute in attributes:
            # print("attribute:", attribute)
            the_uuid = uuid4()
            content = jinja_attribute.render(
                  attribute_uuid=the_uuid
                , attribute_name=attribute["name"]
                , attribute_description=attribute["description"]
                )
            the_attributes.append(json.loads(content))

        # print("Looking for >%s< in current one_on_one datasources:" % source["dataset"], self.one_on_one_data_sources)
        the_uuid = self.get_entity_uuid(source["dataset"], data_sources=self.one_on_one_data_sources)
        content = jinja_attributes.render(
              attributes=json.dumps(the_attributes)
            , entity_uuid=the_uuid
            , version=self.start_time.strftime("%Y.%m.%d")
        )

        # print("1:1 attribute content:", content)
        json_content = json.loads(content)

        directory = path.join(self.lineage_output, self.one_on_one_name)
        makedirs(directory, exist_ok=True)
        file_name = path.join(directory, label + "-" + source["dataset"] + "-1on1_attributes.json")
        with open(file_name, "w", encoding="utf8") as out:
            json.dump(json_content, out, indent=4)

        return result

    def generate_configuration_file(self, out_directory, pattern, data_source):
        """
            The configuration file can be used to generate 1:1 lineage in EDC using the edc_replication_lineage code
            in https://github.com/jacbeekers/Informatica-EDC-REST-API-Samples
            pip3 install "informatica-edc-rest-api-samples>=0.3.67"
        :param out_directory:
        :param pattern: pattern as defined by oneonones.json
        :param data_source: the data source object, used to retrieve src_zone from
        :return: messages.message structure
        """
        result = messages.message["ok"]

        result, jinja_config = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                              , template_name="ref_configuration.jinja")
        if result["code"] != "OK":
            return result

        result, jinja_mapping = self.helper.get_jinja_template(template_directory=self.jinja_templates
                                                               , template_name="edc_source_target_mapping.jinja")
        if result["code"] != "OK":
            return result

        # Note: the following is Informatica EDC naming.
        # TODO: Get the EDC resource name from a configuration file
        mapping_content = jinja_mapping.render(
              source_resource_name=pattern["source_edc_resource"]
            , source_datasource=pattern["source_edc_datasource"]
            , source_container=pattern["source_folder"]
            , target_resource_name=pattern["target_edc_resource"]
            , target_datasource=pattern["target_edc_datasource"]
            , target_container=data_source["src_zone"]
        )

        directory = path.join(out_directory, self.one_on_one_name, "config")
        makedirs(directory, exist_ok=True)
        # print("src_zone is >%s<" % data_source["src_zone"])
        for_zone = data_source["src_zone"].split('/')[0] + "_"
        for_zone += path.basename(data_source["src_zone"])
        filename = "edc_mapping_for_" + for_zone + ".json"
        with open(path.join(directory, filename), "w", encoding="utf8") as config:
            config.write(mapping_content)

        # got the jinja2 templates
        config_content = jinja_config.render(
            edc_source_target_mapping=path.join(directory, filename)
        )
        filename = "config_for_" + for_zone + ".json"
        with open(path.join(directory, filename), "w", encoding="utf8") as config:
            config.write(config_content)

        return result

    def process_ref_zone_file(self, zone_file):
        return messages.message["not_implemented"]

    def process_map_data_source_file(self, data_source_file):
        """
            Generate a csv that can be read by EDC once you've set up a lineage resource to read this csv
            The lineage will show which data sources are connected
        :param data_source_file:
        :return:
        """
        with open(data_source_file, "r") as input:
            src_tgt_data_sources = json.load(input)
        directory = path.join(self.lineage_output, self.one_on_one_name, "lineage")
        makedirs(directory, exist_ok=True)
        csv_file_name = path.join(directory, "lineage_for_data_sources" + ".csv")
        with open(csv_file_name, "w", newline="", encoding="utf8") as f_csv_file:
            col_writer = csv.writer(f_csv_file)
            col_writer.writerow(self.edc_lineage_column_header)
            for src_tgt_data_source in src_tgt_data_sources:
                # print("src_tgt_data_source is:", src_tgt_data_source)
                src_path = path.join("edl://", "FileSystem"
                                     , src_tgt_data_source["source"], src_tgt_data_source["source_dataset"])
                tgt_path = path.join("Rock_Anthems_Playlist://", "FileSystem"
                                     , src_tgt_data_source["target"], src_tgt_data_source["target_dataset"])
                col_writer.writerow(["core.DataSetDataFlow", "", "", src_path, tgt_path])

        return messages.message["ok"]

    def process_map_zone_file(self, zone_file):
        """
            Generate a csv that can be read by EDC once you've setup a lineage resource to read this csv
            The lineage will show which zones (folders) are connected
        :param zone_file:
        :return:
        """
        with open(zone_file, "r") as input:
            src_tgt_zones = json.load(input)
        directory = path.join(self.lineage_output, self.one_on_one_name, "lineage")
        makedirs(directory, exist_ok=True)
        csv_file_name = path.join(directory, "lineage_for_zones" + ".csv")
        with open(csv_file_name, "w", newline="", encoding="utf8") as f_csv_file:
            col_writer = csv.writer(f_csv_file)
            col_writer.writerow(self.edc_lineage_column_header)
            for src_tgt_zone in src_tgt_zones:
                # TODO: Get EDC Resource name from input or config
                # print("src_tgt_zone is:", src_tgt_zone)
                src_path = path.join("edl://", "FileSystem", src_tgt_zone["source"])
                tgt_path = path.join("Rock_Anthems_Playlist://", "FileSystem", src_tgt_zone["target"])
                col_writer.writerow(["core.DataSourceDataFlow", "", "", src_path, tgt_path])

        return messages.message["ok"]

    def main(self):
        if self.result["code"] != "OK":
            print("initialization failure:", self.result)
            return self.result
        else:
            rc = self.process_generated_files()
            return rc



import csv
import json
import os
from datetime import datetime

from openpyxl import load_workbook

from resources import excel_headers


class Excel2Reference:
    start_time = datetime.now()

    def __init__(self, configuration_file="resources/excel2json_config.json"):
        self.excel_location = "resources/datalineage/tmp"
        self.excel_sheet = "excel format"
        self.json_directory = "out/"
        self.json_file_prefix = "generated_"
        self.result = 0
        self.work_book = None
        self.work_sheet = None

        try:
            with open(configuration_file) as config:
                data = json.load(config)
                self.excel_location = data["excel_location"]
                self.excel_sheet = data["excel_sheet"]
                self.json_directory = data["json_directory"]
                if "json_file_prefix" in data:
                    self.json_file_prefix = data["json_file_prefix"]
                else:
                    self.json_file_prefix = self.start_time.isoformat(timespec="microseconds").replace(':', '-')
                if self.json_file_prefix == "TIMESTAMP":
                    self.json_file_prefix = self.start_time.isoformat(timespec="microseconds").replace(':', '-')
                self.json_file_prefix += "--"
        except FileNotFoundError:
            print("Configuration file >" + configuration_file + "< not found.")
            self.result = 1

    def read_excel(self, file=None, sheet=None):
        """
        read a sheet from a provided excel file
        sets:
            self.work_book: a reference to the loaded work_book
            self.work_sheet: a reference to the loaded sheet
        :param file: The Excel file that contains the sheet
        :param sheet: The Excel sheet in the file to be loaded
        :return: result = 2 (one or more parameters not provided (None)), 3 (FileNotFound), 10 (IOError), 0 = ok
        """
        path = ""
        result = 0
        if file is None or sheet is None:
            return 2
        self.work_book = None
        self.work_sheet = None
        try:
            path = self.excel_location + "/" + file
            self.work_book = load_workbook(path)
            self.work_sheet = self.work_book[sheet]
        except FileNotFoundError as e:
            print("Could not find/read Excel >" + path + "<. Error: " + str(e))
            return 3
        except IOError as e:
            print("IOError loading Excel file >" + path + "<. Error: " + str(e))
            return 10

        return result

    def create_output_files(self):
        result = 0
        info_datasets = []
        # list of source zones
        source_zone_column = "zone"
        source_zones = self.get_distinct(source_zone_column)
        result += self.out_distinct("REF_-_zones-source" + source_zone_column, source_zones)

        # list of target zones
        target_zone_column = "tgtzone"
        target_zones = self.get_distinct(target_zone_column)
        result += self.out_distinct("REF_-_zones-target" + target_zone_column, target_zones)

        # source dataset list
        source_dataset_column = "dataset"
        source_datasets = self.get_distinct(source_dataset_column)
        # get additional info
        info_source_datasets = self.get_dataset_info(source_datasets, "dataset")
        print("dataset info collected for >%d< source datasets" % len(info_source_datasets))
        info_datasets.extend(info_source_datasets)
        result += self.out_distinct_combined("REF_-_data-sources-source" + source_dataset_column, info_source_datasets)

        # target dataset list
        target_dataset_column = "tgtdataset"
        target_datasets = self.get_distinct(target_dataset_column)
        # get additional info
        info_target_datasets = self.get_dataset_info(target_datasets, "tgtdataset")
        print("dataset info collected for >%d< target datasets" % len(info_target_datasets))
        info_datasets.extend(info_target_datasets)
        print("In total: dataset info collected for >%d< datasets" % len(info_datasets))
        result += self.out_distinct_combined("REF_-_data-sources-target" + target_dataset_column, info_target_datasets)

        # which source zone has which target zone
        src_zone, row = self.search_value_in_row_index(search_string=source_zone_column, row=1)
        tgt_zone, row = self.search_value_in_row_index(search_string=target_zone_column, row=1)
        src_tgt_zones = self.get_distinct_combined(src_zone, tgt_zone)
        result += self.out_distinct_combined("MAP_-_zones-source2target"
                                             + source_zone_column + '_' + target_zone_column, src_tgt_zones)

        # which source dataset in which source zone has which target dataset in which target zone
        src_zone, row = self.search_value_in_row_index(search_string=source_zone_column, row=1)
        tgt_zone, row = self.search_value_in_row_index(search_string=target_zone_column, row=1)
        src_dataset, row = self.search_value_in_row_index(search_string=source_dataset_column, row=1)
        tgt_dataset, row = self.search_value_in_row_index(search_string=target_dataset_column, row=1)
        src_tgt_zones_datasets = self.get_distinct_combined(source_zone=src_zone
                                                            , target_zone=tgt_zone
                                                            , source_dataset=src_dataset
                                                            , target_dataset=tgt_dataset
                                                            )
        result += self.out_distinct_combined("MAP_-_data-sources-source2target"
                                             + source_zone_column + "." + source_dataset_column
                                             + "_" + target_zone_column + "." + target_dataset_column
                                             , src_tgt_zones_datasets)
        for dataset in info_datasets:
            entities = []
            attributes = self.get_dataset_attributes(dataset_info=dataset)
            entity = dict(is_source=dataset["is_source"]
                          , dataset=dataset["dataset"]
                          , src_zone=dataset["src_zone"]
                          , attributes=attributes)
            entities.append(entity)
            file_name = os.path.join(self.json_directory, self.json_file_prefix + "ATTRIB_-_" + dataset["dataset"])
            self.write_dict_to_file(out_file=file_name, result_list=entities)

        return result

    def get_dataset_attributes(self, dataset_info):
        # print(dataset_info)
        src_dataset_column = self.search_value_in_row_index(
            search_string=excel_headers.source_dataset_name)[0]
        src_attribute_column = self.search_value_in_row_index(
            search_string=excel_headers.source_attribute)[0]
        src_attribute_description = self.search_value_in_row_index(
            search_string=excel_headers.source_attribute_description)[0]
        tgt_dataset_column = self.search_value_in_row_index(
            search_string=excel_headers.target_dataset_name)[0]
        tgt_attribute_column = self.search_value_in_row_index(
            search_string=excel_headers.target_attribute)[0]
        tgt_attribute_description = self.search_value_in_row_index(
            search_string=excel_headers.target_attribute_description)[0]
        if dataset_info["is_source"] == "True":
            attributes = self.search_value_in_column_index(search_string=dataset_info["dataset"]
                                                           , column=src_dataset_column
                                                           , give_column=src_attribute_column
                                                           , give_column2=src_attribute_description
                                                           , name_column1="attribute_name"
                                                           , name_column2="attribute_description"
                                                           , give_list=True)
        else:
            # print("not is_source. search_string >%s< for column >%d<. Asked for >%d< and >%d<."
            #      % (dataset_info["dataset"], tgt_dataset_column, tgt_attribute_column, tgt_attribute_description))
            attributes = self.search_value_in_column_index(search_string=dataset_info["dataset"]
                                                           , column=tgt_dataset_column
                                                           , give_column=tgt_attribute_column
                                                           , give_column2=tgt_attribute_description
                                                           , name_column1="attribute_name"
                                                           , name_column2="attribute_description"
                                                           , give_list=True)
        print("attributes:", attributes)
        return attributes

    def get_dataset_info(self, datasets, source_or_target):
        info_datasets = []
        print("source_or_target is:", source_or_target)
        ds_column = self.search_value_in_row_index(search_string=source_or_target)[0]
        description_column = self.search_value_in_row_index(search_string=excel_headers.source_dataset_description)[0]
        id_column = self.search_value_in_row_index(search_string=excel_headers.source_identifier)[0]
        app_column = self.search_value_in_row_index(search_string=excel_headers.application_identifier)[0]
        src_zone_column = self.search_value_in_row_index(search_string=excel_headers.source_zone)[0]
        tgt_zone_column = self.search_value_in_row_index(search_string=excel_headers.target_zone)[0]

        for dataset in datasets:
            description = self.search_value_in_column_index(search_string=dataset, column=ds_column
                                                            , give_column=description_column)
            if description is None:
                description = ""
            the_id = self.search_value_in_column_index(search_string=dataset, column=ds_column
                                                       , give_column=id_column)
            if the_id is None:
                the_id = ""
            the_id = str(the_id)
            app = self.search_value_in_column_index(search_string=dataset, column=ds_column
                                                    , give_column=app_column)
            if app is None:
                app = ""

            is_source = "False"
            if source_or_target == "dataset":
                source_zone = self.search_value_in_column_index(search_string=dataset, column=ds_column
                                                                , give_column=src_zone_column)
                is_source = "True"
            else:
                source_zone = ""
            if source_zone is None:
                source_zone = ""

            if source_or_target == "tgtdataset":
                target_zone = self.search_value_in_column_index(search_string=dataset, column=ds_column
                                                                , give_column=tgt_zone_column)
                is_source = "False"
            else:
                target_zone = ""
            if target_zone is None:
                target_zone = ""

            info_datasets.append(dict(is_source=is_source, dataset=dataset, description=description, id=the_id, app=app
                                      , src_zone=source_zone, tgt_zone=target_zone))

        return info_datasets

    def get_distinct_combined(self, source_zone=None, target_zone=None, source_dataset=None, target_dataset=None):
        if source_zone is None or target_zone is None:
            return None
        if source_dataset is not None and target_dataset is None:
            return None
        if source_dataset is None and target_dataset is not None:
            return None

        values = []
        id_column = self.search_value_in_row_index(search_string=excel_headers.source_identifier)[0]
        app_column = self.search_value_in_row_index(search_string=excel_headers.application_identifier)[0]
        src_zone_column = self.search_value_in_row_index(search_string=excel_headers.source_zone)[0]
        tgt_zone_column = self.search_value_in_row_index(search_string=excel_headers.target_zone)[0]

        for i in range(2, self.work_sheet.max_row + 1):
            source_value = self.work_sheet.cell(row=i, column=source_zone).value
            target_value = self.work_sheet.cell(row=i, column=target_zone).value
            if source_dataset is None:
                combined = dict(source=source_value, target=target_value)
                combined_for_list = combined
                if combined in values:
                    pass
                else:
                    if combined_for_list["target"] is not None:
                        values.append(combined_for_list)
            else:
                source_dataset_value = self.work_sheet.cell(row=i, column=source_dataset).value
                target_dataset_value = self.work_sheet.cell(row=i, column=target_dataset).value
                src_zone_excel = self.work_sheet.cell(row=i, column=src_zone_column).value
                if src_zone_excel is not None:
                    src_layer, src_zone, src_area, src_system = self.split_zone_excel_value(src_zone_excel)
                else:
                    src_layer = None
                    src_zone = None
                    src_area = None
                    src_system = None
                tgt_zone_excel = self.work_sheet.cell(row=i, column=tgt_zone_column).value
                if tgt_zone_excel is not None:
                    tgt_layer, tgt_zone, tgt_area, tgt_system = self.split_zone_excel_value(tgt_zone_excel)
                else:
                    tgt_layer = None
                    tgt_zone = None
                    tgt_area = None
                    tgt_system = None
                # combined = dict(source=source_value
                #                , source_dataset=source_dataset_value
                #                , target=target_value
                #                , target_dataset=target_dataset_value
                #                , src_zone=src_zone
                #                , tgt_zone=tgt_zone
                #                )
                combined = dict(
                      source=source_value
                    , source_dataset=source_dataset_value
                    , target=target_value
                    , target_dataset=target_dataset_value
                    , application_id=self.work_sheet.cell(row=i, column=app_column).value
                    , dataset_id=str(self.work_sheet.cell(row=i, column=id_column).value)
                    , src_layer = src_layer
                    , src_zone=src_zone
                    , src_area=src_area
                    , src_system=src_system
                    , tgt_layer = tgt_layer
                    , tgt_zone=tgt_zone
                    , tgt_area=tgt_area
                    , tgt_system=tgt_system
                )
                if combined in values:
                    pass
                else:
                    if combined["target"] is not None and combined["tgt_zone"] is not None:
                        values.append(combined)
        return values

    def split_zone_excel_value(self, zone_from_excel):
        layer = "unknown"
        zone = "unknown"
        area = "unknown"
        system = "unknown"

        fields = zone_from_excel.split('/', 4)
        for field in fields:
            if "_layer" in field:
                layer = field
            if "_zone" in field:
                zone = field
        if len(fields) >= 3:
            area = fields[2]
        if len(fields) >= 4:
            system = fields[3]

        return layer, zone, area, system

    def out_distinct(self, column_name=None, result_list=None):
        if column_name is None or result_list is None:
            return None
        print(result_list)
        out_file = os.path.join(self.json_directory, self.json_file_prefix + column_name + ".csv")
        return self.write_list_to_file(out_file, result_list)

    def out_distinct_combined(self, filename, source_targets=None):
        if source_targets is None:
            return 6
        print(source_targets)
        # method will add extension
        out_file = os.path.join(self.json_directory, self.json_file_prefix + filename)
        return self.write_dict_to_file(out_file, source_targets)

    def get_distinct(self, column_name=None):
        if column_name is None:
            return None
        values = []

        column, row = self.search_value_in_row_index(search_string=column_name, row=1)

        for i in range(2, self.work_sheet.max_row + 1):
            if self.work_sheet.cell(row=i, column=column).value in values:
                pass
            else:
                if self.work_sheet.cell(row=i, column=column).value is not None:
                    values.append(self.work_sheet.cell(row=i, column=column).value)
        return values

    def search_value_in_row_index(self, search_string, row=1):
        """
        Thankfully copied from
        https://stackoverflow.com/questions/50491839/python-openpyxl-find-strings-in-column-and-return-row-number
        """
        for cell in self.work_sheet[row]:
            if cell.value == search_string:
                return cell.column, row
        return None, row

    def search_value_in_column_index(self, search_string, column, give_column, give_column2=None,
                                     name_column1=None, name_column2=None, give_list=False):
        # TODO: Make more generic and flexible
        # Note: give_column2 is only valid in combination with give_list=True

        # print("Searching for", search_string, "in column", column)
        values = []
        for row in self.work_sheet.iter_rows(column):
            for cell in row:
                # print(cell.row, cell.column, cell.value)
                if cell.value == search_string:
                    # print(cell.value)
                    if not give_list:
                        # we found it and we should not provide an entire list, so return to caller
                        return str(self.work_sheet.cell(row=cell.row, column=give_column).value).replace('"','\"').replace('\\n',' ')
                    else:
                        if give_column2 is None:
                            value = self.work_sheet.cell(row=cell.row, column=give_column).value
                            if value is None:
                                value = ""
                            else:
                                value = value.replace('"', '\"')
                            values.append(value)
                        else:
                            value1 = self.work_sheet.cell(row=cell.row, column=give_column).value
                            if value1 is None:
                                value1 = ""
                            else:
                                value1 = value1.replace('"', '\"')
                            value2 = self.work_sheet.cell(row=cell.row, column=give_column2).value
                            if value2 is None:
                                value2 = ""
                            else:
                                value2 = value2.replace('"', '\"')
                            col_dict = {name_column1: value1
                                , name_column2: value2}
                            values.append(col_dict)
        if give_list:
            nr = 0
            out_list = []
            for value in values:
                already_exists = False
                for out in out_list:
                    if value["attribute_name"] == out["attribute_name"]:
                        already_exists = True
                        break
                if already_exists:
                    pass
                else:
                    out_list.append(value)
            return out_list
        #    return list(set(values))
        else:
            return None

    def write_list_to_file(self, out_file, result_list):
        try:
            with open(out_file, "w", encoding="utf-8", newline='') as csv_file:
                for item in result_list:
                    if item is not None:
                        csv_file.write(item.replace('/', ';') + "\n")
        except IOError:
            print("Error creating or writing to output file >" + out_file + "<.")
            return 4
        return 0

    def write_dict_to_file(self, out_file, result_list):
        # csv
        file_name = out_file + ".csv"
        try:
            keys = result_list[0].keys()
            with open(file_name, "w", encoding="utf8", newline='') as csv_file:
                dict_writer = csv.DictWriter(csv_file, keys, quotechar='"', quoting=csv.QUOTE_NONNUMERIC)
                dict_writer.writeheader()
                dict_writer.writerows(result_list)
        except IOError:
            print("Error creating or writing dictionary to csv output file >" + file_name + "<.")
            return 8
        # json
        file_name = out_file + ".json"
        try:
            with open(file_name, "w", encoding="utf8", newline='') as json_file:
                json_file.write(json.dumps(result_list, indent=4))
        except IOError:
            print("Error creating or writing dictionary to json output file >" + file_name + "<.")
            return 9
        return 0

    def main(self, excel=None):
        if self.result == 1:
            return self.result

        result = self.read_excel(excel, self.excel_sheet)
        if result != 0:
            return result

        result = self.create_output_files()
        return result


if __name__ == "__main__":
    excel2ref = Excel2Reference(configuration_file="resources/excel2json_config.json")
    result = excel2ref.main(excel="tryout.xlsx")
    exit(result)



from excel2meta_interface.utils import messages
# import informatica_edc_rest_api_samples
from edc_rest_api.metadata_utilities import load_json_metadata
import glob


class UploadLineageToEDC:
    def __init__(self):
        self.result = messages.message["ok"]
        self.config_directory = "lineage_output/one-on-one/config/"

    def upload_lineage(self):
        print("Uploading lineage to EDC with config files in >%s<" % self.config_directory)
        for file in glob.glob(self.config_directory + "config_for_*.json"):
            print("Lineage processing with configuration file >%s<" % file)
            load_json = load_json_metadata.ConvertJSONtoEDCLineage(configuration_file=file)
            load_json.main(ignore_metafile_creation=True)


if __name__ == "__main__":
    UploadLineageToEDC().upload_lineage()



message = {
    "ok": {
        "code": "OK"
        , "message": "No errors encountered"
        , "level": "INFO"
    },
    "not_implemented": {
        "code": "EMI-GEN-000"
        , "message": "Function not yet implemented. Please contact the development team and mention code 'EMI-GEN-000'."
        , "level": "ERROR"
    },
    "not_found": {
        "code": "EMI-GEN-001"
        , "message": "Could not find what we were looking for"
        , "level": "ERROR"
    },
    "os_error": {
        "code": "EMI-GEN-002"
        , "message": "An OS error occurred"
        , "level": "ERROR"
    },
    "ignore": {
        "code": "EMI-GEN-003"
        , "message": "Entry has been ignored"
        , "level": "DEBUG"
    },
    "undetermined": {
        "code": "EMI-GEN-004"
        ,
        "message": "Undetermined result. This should not happen. Please contact the development team and mention code 'EMI-GEN-004'."
        , "level": "ERROR"
    },
    "main_config_not_found": {
        "code": "EMI-GEN-005"
        , "message": "Main configuration file not found."
        , "level": "FATAL"
    },
    "jinja_template_name_not_provided": {
        "code": "EMI-JIN-001"
        , "message": "Internal error: Jinja template name not provided"
        , "level": "ERROR"
    },
    "jinja_template_not_found": {
        "code": "EMI-JIN-002"
        , "message": "Jinja template not found."
        , "level": "ERROR"
    }
}
